# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .. import models as _models
from .._configuration import (
    HeaderParamClientConfiguration,
    MixedParamsClientConfiguration,
    MultipleParamsClientConfiguration,
    ParamAliasClientConfiguration,
    PathParamClientConfiguration,
    QueryParamClientConfiguration,
)
from .._utils.model_base import SdkJSONEncoder, _deserialize
from .._utils.serialization import Serializer
from .._utils.utils import ClientMixinABC

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_header_param_with_query_request(*, id: str, name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/header-param/with-query"

    # Construct parameters
    _params["id"] = _SERIALIZER.query("id", id, "str")

    # Construct headers
    _headers["name"] = _SERIALIZER.header("name", name, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_header_param_with_body_request(*, name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/header-param/with-body"

    # Construct headers
    _headers["name"] = _SERIALIZER.header("name", name, "str")
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_multiple_params_with_query_request(*, id: str, name: str, region: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/multiple-params/with-query"

    # Construct parameters
    _params["region"] = _SERIALIZER.query("region", region, "str")
    _params["id"] = _SERIALIZER.query("id", id, "str")

    # Construct headers
    _headers["name"] = _SERIALIZER.header("name", name, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_multiple_params_with_body_request(*, name: str, region: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/multiple-params/with-body"

    # Construct parameters
    _params["region"] = _SERIALIZER.query("region", region, "str")

    # Construct headers
    _headers["name"] = _SERIALIZER.header("name", name, "str")
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_mixed_params_with_query_request(*, region: str, id: str, name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/mixed-params/with-query"

    # Construct parameters
    _params["region"] = _SERIALIZER.query("region", region, "str")
    _params["id"] = _SERIALIZER.query("id", id, "str")

    # Construct headers
    _headers["name"] = _SERIALIZER.header("name", name, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_mixed_params_with_body_request(*, region: str, name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/mixed-params/with-body"

    # Construct parameters
    _params["region"] = _SERIALIZER.query("region", region, "str")

    # Construct headers
    _headers["name"] = _SERIALIZER.header("name", name, "str")
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_path_param_with_query_request(blob_name: str, *, format: Optional[str] = None, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/path/{blobName}/with-query"
    path_format_arguments = {
        "blobName": _SERIALIZER.url("blob_name", blob_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    if format is not None:
        _params["format"] = _SERIALIZER.query("format", format, "str")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_path_param_get_standalone_request(blob_name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/path/{blobName}/get-standalone"
    path_format_arguments = {
        "blobName": _SERIALIZER.url("blob_name", blob_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


def build_path_param_delete_standalone_request(  # pylint: disable=name-too-long
    blob_name: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/path/{blobName}"
    path_format_arguments = {
        "blobName": _SERIALIZER.url("blob_name", blob_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    return HttpRequest(method="DELETE", url=_url, **kwargs)


def build_param_alias_with_aliased_name_request(  # pylint: disable=name-too-long
    blob_name: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/param-alias/{blob}/with-aliased-name"
    path_format_arguments = {
        "blob": _SERIALIZER.url("blob_name", blob_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    return HttpRequest(method="GET", url=_url, **kwargs)


def build_param_alias_with_original_name_request(  # pylint: disable=name-too-long
    blob_name: str, **kwargs: Any
) -> HttpRequest:
    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/param-alias/{blobName}/with-original-name"
    path_format_arguments = {
        "blobName": _SERIALIZER.url("blob_name", blob_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    return HttpRequest(method="GET", url=_url, **kwargs)


def build_query_param_with_query_request(*, blob_name: str, format: Optional[str] = None, **kwargs: Any) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/query/with-query"

    # Construct parameters
    _params["blobName"] = _SERIALIZER.query("blob_name", blob_name, "str")
    if format is not None:
        _params["format"] = _SERIALIZER.query("format", format, "str")

    return HttpRequest(method="GET", url=_url, params=_params, **kwargs)


def build_query_param_get_standalone_request(*, blob_name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/query/get-standalone"

    # Construct parameters
    _params["blobName"] = _SERIALIZER.query("blob_name", blob_name, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_query_param_delete_standalone_request(  # pylint: disable=name-too-long
    *, blob_name: str, **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    # Construct URL
    _url = "/azure/client-generator-core/client-initialization/default/query/delete-resource"

    # Construct parameters
    _params["blobName"] = _SERIALIZER.query("blob_name", blob_name, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, **kwargs)


class _HeaderParamClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], HeaderParamClientConfiguration]
):

    @distributed_trace
    def with_query(self, *, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """with_query.

        :keyword id: Required.
        :paramtype id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_header_param_with_query_request(
            id=id,
            name=self._config.name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    def with_body(self, body: _models.Input, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: ~specs.azure.clientgenerator.core.clientinitialization.default.models.Input
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def with_body(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def with_body(self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def with_body(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.Input, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """with_body.

        :param body: Is one of the following types: Input, JSON, IO[bytes] Required.
        :type body: ~specs.azure.clientgenerator.core.clientinitialization.default.models.Input or JSON
         or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_header_param_with_body_request(
            name=self._config.name,
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class _MultipleParamsClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], MultipleParamsClientConfiguration]
):

    @distributed_trace
    def with_query(self, *, id: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """with_query.

        :keyword id: Required.
        :paramtype id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_multiple_params_with_query_request(
            id=id,
            name=self._config.name,
            region=self._config.region,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    def with_body(self, body: _models.Input, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: ~specs.azure.clientgenerator.core.clientinitialization.default.models.Input
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def with_body(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def with_body(self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def with_body(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.Input, JSON, IO[bytes]], **kwargs: Any
    ) -> None:
        """with_body.

        :param body: Is one of the following types: Input, JSON, IO[bytes] Required.
        :type body: ~specs.azure.clientgenerator.core.clientinitialization.default.models.Input or JSON
         or IO[bytes]
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_multiple_params_with_body_request(
            name=self._config.name,
            region=self._config.region,
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class _MixedParamsClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], MixedParamsClientConfiguration]
):

    @distributed_trace
    def with_query(  # pylint: disable=inconsistent-return-statements
        self, *, region: str, id: str, **kwargs: Any
    ) -> None:
        """with_query.

        :keyword region: Required.
        :paramtype region: str
        :keyword id: Required.
        :paramtype id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_mixed_params_with_query_request(
            region=region,
            id=id,
            name=self._config.name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    def with_body(
        self, body: _models.WithBodyRequest, *, region: str, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """with_body.

        :param body: Required.
        :type body:
         ~specs.azure.clientgenerator.core.clientinitialization.default.models.WithBodyRequest
        :keyword region: Required.
        :paramtype region: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def with_body(self, body: JSON, *, region: str, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: JSON
        :keyword region: Required.
        :paramtype region: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def with_body(self, body: IO[bytes], *, region: str, content_type: str = "application/json", **kwargs: Any) -> None:
        """with_body.

        :param body: Required.
        :type body: IO[bytes]
        :keyword region: Required.
        :paramtype region: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def with_body(  # pylint: disable=inconsistent-return-statements
        self, body: Union[_models.WithBodyRequest, JSON, IO[bytes]], *, region: str, **kwargs: Any
    ) -> None:
        """with_body.

        :param body: Is one of the following types: WithBodyRequest, JSON, IO[bytes] Required.
        :type body:
         ~specs.azure.clientgenerator.core.clientinitialization.default.models.WithBodyRequest or JSON
         or IO[bytes]
        :keyword region: Required.
        :paramtype region: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_mixed_params_with_body_request(
            region=region,
            name=self._config.name,
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class _PathParamClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], PathParamClientConfiguration]
):

    @distributed_trace
    def with_query(  # pylint: disable=inconsistent-return-statements
        self, *, format: Optional[str] = None, **kwargs: Any
    ) -> None:
        """with_query.

        :keyword format: Default value is None.
        :paramtype format: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_param_with_query_request(
            blob_name=self._config.blob_name,
            format=format,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def get_standalone(self, **kwargs: Any) -> _models.BlobProperties:
        """get_standalone.

        :return: BlobProperties. The BlobProperties is compatible with MutableMapping
        :rtype: ~specs.azure.clientgenerator.core.clientinitialization.default.models.BlobProperties
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.BlobProperties] = kwargs.pop("cls", None)

        _request = build_path_param_get_standalone_request(
            blob_name=self._config.blob_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.BlobProperties, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def delete_standalone(self, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """delete_standalone.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_path_param_delete_standalone_request(
            blob_name=self._config.blob_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class _ParamAliasClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], ParamAliasClientConfiguration]
):

    @distributed_trace
    def with_aliased_name(self, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """with_aliased_name.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_param_alias_with_aliased_name_request(
            blob_name=self._config.blob_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def with_original_name(self, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """with_original_name.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_param_alias_with_original_name_request(
            blob_name=self._config.blob_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class _QueryParamClientOperationsMixin(
    ClientMixinABC[PipelineClient[HttpRequest, HttpResponse], QueryParamClientConfiguration]
):

    @distributed_trace
    def with_query(  # pylint: disable=inconsistent-return-statements
        self, *, format: Optional[str] = None, **kwargs: Any
    ) -> None:
        """with_query.

        :keyword format: Default value is None.
        :paramtype format: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_param_with_query_request(
            blob_name=self._config.blob_name,
            format=format,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def get_standalone(self, **kwargs: Any) -> _models.BlobProperties:
        """get_standalone.

        :return: BlobProperties. The BlobProperties is compatible with MutableMapping
        :rtype: ~specs.azure.clientgenerator.core.clientinitialization.default.models.BlobProperties
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.BlobProperties] = kwargs.pop("cls", None)

        _request = build_query_param_get_standalone_request(
            blob_name=self._config.blob_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.BlobProperties, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def delete_standalone(self, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """delete_standalone.

        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_query_param_delete_standalone_request(
            blob_name=self._config.blob_name,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhcivm.aio import AzureStackHCIVmClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIVmVirtualMachineInstancesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIVmClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_get(self, resource_group):
        response = await self.client.virtual_machine_instances.get(
            resource_uri="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_create_or_update(
                resource_uri="str",
                resource={
                    "extendedLocation": {"name": "str", "type": "str"},
                    "id": "str",
                    "identity": {
                        "type": "str",
                        "principalId": "str",
                        "tenantId": "str",
                        "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                    },
                    "name": "str",
                    "properties": {
                        "createFromLocal": bool,
                        "guestAgentInstallStatus": {
                            "agentVersion": "str",
                            "errorDetails": [
                                {
                                    "additionalInfo": [{"info": {}, "type": "str"}],
                                    "code": "str",
                                    "details": [...],
                                    "message": "str",
                                    "target": "str",
                                }
                            ],
                            "lastStatusChange": "2020-02-20 00:00:00",
                            "status": "str",
                            "vmUuid": "str",
                        },
                        "hardwareProfile": {
                            "dynamicMemoryConfig": {
                                "maximumMemoryMB": 0,
                                "minimumMemoryMB": 0,
                                "targetMemoryBuffer": 0,
                            },
                            "memoryMB": 0,
                            "processors": 0,
                            "virtualMachineGPUs": [{"assignmentType": "str", "gpuName": "str", "partitionSizeMB": 0}],
                            "vmSize": "str",
                        },
                        "hostNodeIpAddress": "str",
                        "hostNodeName": "str",
                        "httpProxyConfig": {
                            "httpProxy": "str",
                            "httpsProxy": "str",
                            "noProxy": ["str"],
                            "trustedCa": "str",
                        },
                        "hyperVVmId": "str",
                        "instanceView": {
                            "vmAgent": {
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                                "vmConfigAgentVersion": "str",
                            }
                        },
                        "networkProfile": {"networkInterfaces": [{"id": "str"}]},
                        "osProfile": {
                            "adminPassword": "str",
                            "adminUsername": "str",
                            "computerName": "str",
                            "linuxConfiguration": {
                                "disablePasswordAuthentication": bool,
                                "provisionVMAgent": bool,
                                "provisionVMConfigAgent": bool,
                                "ssh": {"publicKeys": [{"keyData": "str", "path": "str"}]},
                            },
                            "windowsConfiguration": {
                                "enableAutomaticUpdates": bool,
                                "provisionVMAgent": bool,
                                "provisionVMConfigAgent": bool,
                                "ssh": {"publicKeys": [{"keyData": "str", "path": "str"}]},
                                "timeZone": "str",
                            },
                        },
                        "placementProfile": {"strictPlacementPolicy": bool, "zone": "str"},
                        "provisioningState": "str",
                        "resourceUid": "str",
                        "securityProfile": {
                            "enableTPM": bool,
                            "securityType": "str",
                            "uefiSettings": {"secureBootEnabled": bool},
                        },
                        "status": {
                            "errorCode": "str",
                            "errorMessage": "str",
                            "powerState": "str",
                            "provisioningStatus": {"operationId": "str", "status": "str"},
                        },
                        "storageProfile": {
                            "dataDisks": [{"id": "str"}],
                            "imageReference": {"id": "str"},
                            "osDisk": {
                                "id": "str",
                                "managedDisk": {"securityProfile": {"securityEncryptionType": "str"}},
                                "osType": "str",
                            },
                            "vmConfigStoragePathId": "str",
                        },
                        "vmId": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_update(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_update(
                resource_uri="str",
                properties={
                    "identity": {"principalId": "str", "tenantId": "str", "type": "SystemAssigned"},
                    "properties": {
                        "hardwareProfile": {
                            "memoryMB": 0,
                            "processors": 0,
                            "virtualMachineGPUs": [{"assignmentType": "str", "gpuName": "str", "partitionSizeMB": 0}],
                            "vmSize": "str",
                        },
                        "networkProfile": {"networkInterfaces": [{"id": "str"}]},
                        "osProfile": {
                            "computerName": "str",
                            "linuxConfiguration": {"provisionVMAgent": bool, "provisionVMConfigAgent": bool},
                            "windowsConfiguration": {"provisionVMAgent": bool, "provisionVMConfigAgent": bool},
                        },
                        "storageProfile": {"dataDisks": [{"id": "str"}]},
                    },
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_delete(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_delete(
                resource_uri="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_list(self, resource_group):
        response = self.client.virtual_machine_instances.list(
            resource_uri="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_start(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_start(
                resource_uri="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_stop(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_stop(
                resource_uri="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_restart(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_restart(
                resource_uri="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_pause(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_pause(
                resource_uri="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_virtual_machine_instances_begin_save(self, resource_group):
        response = await (
            await self.client.virtual_machine_instances.begin_save(
                resource_uri="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import DatetimePreparer
from testpreparer_async import DatetimeClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDatetimeQueryOperationsAsync(DatetimeClientTestBaseAsync):
    @DatetimePreparer()
    @recorded_by_proxy_async
    async def test_query_default(self, datetime_endpoint):
        client = self.create_async_client(endpoint=datetime_endpoint)
        response = await client.query.default(
            value="2020-02-20 00:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @DatetimePreparer()
    @recorded_by_proxy_async
    async def test_query_rfc3339(self, datetime_endpoint):
        client = self.create_async_client(endpoint=datetime_endpoint)
        response = await client.query.rfc3339(
            value="2020-02-20 00:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @DatetimePreparer()
    @recorded_by_proxy_async
    async def test_query_rfc7231(self, datetime_endpoint):
        client = self.create_async_client(endpoint=datetime_endpoint)
        response = await client.query.rfc7231(
            value="2020-02-20 00:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @DatetimePreparer()
    @recorded_by_proxy_async
    async def test_query_unix_timestamp(self, datetime_endpoint):
        client = self.create_async_client(endpoint=datetime_endpoint)
        response = await client.query.unix_timestamp(
            value="2020-02-20 00:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @DatetimePreparer()
    @recorded_by_proxy_async
    async def test_query_unix_timestamp_array(self, datetime_endpoint):
        client = self.create_async_client(endpoint=datetime_endpoint)
        response = await client.query.unix_timestamp_array(
            value=["2020-02-20 00:00:00"],
        )

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------


"""
DESCRIPTION:
    This sample demonstrates how to use agent operations with the
    Azure AI Search tool from the Azure agents service using an asynchronous client.

PREREQUISITES:
    You will need an Azure AI Search Resource connected to your Foundry project.
    If you already have one, you must create an agent that can use an existing Azure AI Search index:
    https://learn.microsoft.com/azure/ai-services/agents/how-to/tools/azure-ai-search?tabs=azurecli%2Cpython&pivots=overview-azure-ai-search

    If you do not already have an agent Setup with an Azure AI Search resource, follow the guide for a Standard agent setup:
    https://learn.microsoft.com/azure/ai-services/agents/quickstart?pivots=programming-language-python-azure

USAGE:
    python sample_agents_azure_ai_search_async.py

    Before running the sample:

    pip install azure-ai-projects azure-ai-agents azure-identity aiohttp

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
"""

import asyncio
import os
from azure.ai.projects.aio import AIProjectClient
from azure.ai.projects.models import ConnectionType
from azure.identity.aio import DefaultAzureCredential
from azure.ai.agents.models import AzureAISearchQueryType, AzureAISearchTool, ListSortOrder, MessageRole


async def main() -> None:

    credential = DefaultAzureCredential()

    async with credential:

        project_client = AIProjectClient(
            endpoint=os.environ["PROJECT_ENDPOINT"],
            credential=credential,
        )

        async with project_client:

            conn_id = (await project_client.connections.get_default(ConnectionType.AZURE_AI_SEARCH)).id
            print(conn_id)

            # Initialize agent AI search tool and add the search index connection id
            ai_search = AzureAISearchTool(
                index_connection_id=conn_id,
                index_name="sample_index",
                query_type=AzureAISearchQueryType.SIMPLE,
                top_k=3,
                filter="",
            )

            agent = await project_client.agents.create_agent(
                model=os.environ["MODEL_DEPLOYMENT_NAME"],
                name="my-agent",
                instructions="You are a helpful agent",
                tools=ai_search.definitions,
                tool_resources=ai_search.resources,
            )
            print(f"Created agent, ID: {agent.id}")

            # Create thread for communication
            thread = await project_client.agents.threads.create()
            print(f"Created thread, ID: {thread.id}")

            # Create message to thread
            message = await project_client.agents.messages.create(
                thread_id=thread.id,
                role="user",
                content="What is the temperature rating of the cozynights sleeping bag?",
            )
            print(f"Created message, ID: {message.id}")

            # Create and process agent run in thread with tools
            run = await project_client.agents.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
            print(f"Run finished with status: {run.status}")

            if run.status == "failed":
                print(f"Run failed: {run.last_error}")

            # Fetch run steps to get the details of the agent run
            run_steps = project_client.agents.run_steps.list(thread_id=thread.id, run_id=run.id)
            async for step in run_steps:
                print(f"Step {step['id']} status: {step['status']}")
                step_details = step.get("step_details", {})
                tool_calls = step_details.get("tool_calls", [])

                if tool_calls:
                    print("  Tool calls:")
                    for call in tool_calls:
                        print(f"    Tool Call ID: {call.get('id')}")
                        print(f"    Type: {call.get('type')}")

                        azure_ai_search_details = call.get("azure_ai_search", {})
                        if azure_ai_search_details:
                            print(f"    azure_ai_search input: {azure_ai_search_details.get('input')}")
                            print(f"    azure_ai_search output: {azure_ai_search_details.get('output')}")
                print()  # add an extra newline between steps

            # Delete the agent when done
            await project_client.agents.delete_agent(agent.id)
            print("Deleted agent")

            # Fetch and log all messages
            messages = project_client.agents.messages.list(thread_id=thread.id, order=ListSortOrder.ASCENDING)
            async for message in messages:
                if message.role == MessageRole.AGENT and message.url_citation_annotations:
                    placeholder_annotations = {
                        annotation.text: f" [see {annotation.url_citation.title}] ({annotation.url_citation.url})"
                        for annotation in message.url_citation_annotations
                    }
                    for message_text in message.text_messages:
                        message_str = message_text.text.value
                        for k, v in placeholder_annotations.items():
                            message_str = message_str.replace(k, v)
                        print(f"{message.role}: {message_str}")
                else:
                    for message_text in message.text_messages:
                        print(f"{message.role}: {message_text.text.value}")


if __name__ == "__main__":
    asyncio.run(main())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.appcontainers import ContainerAppsAPIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestContainerAppsAPIAppResiliencyOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ContainerAppsAPIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_app_resiliency_create_or_update(self, resource_group):
        response = self.client.app_resiliency.create_or_update(
            resource_group_name=resource_group.name,
            app_name="str",
            name="str",
            resiliency_envelope={
                "circuitBreakerPolicy": {"consecutiveErrors": 0, "intervalInSeconds": 0, "maxEjectionPercent": 0},
                "httpConnectionPool": {"http1MaxPendingRequests": 0, "http2MaxRequests": 0},
                "httpRetryPolicy": {
                    "errors": ["str"],
                    "headers": [
                        {
                            "exactMatch": "str",
                            "header": "str",
                            "prefixMatch": "str",
                            "regexMatch": "str",
                            "suffixMatch": "str",
                        }
                    ],
                    "httpStatusCodes": [0],
                    "initialDelayInMilliseconds": 0,
                    "maxIntervalInMilliseconds": 0,
                    "maxRetries": 0,
                },
                "id": "str",
                "name": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tcpConnectionPool": {"maxConnections": 0},
                "tcpRetryPolicy": {"maxConnectAttempts": 0},
                "timeoutPolicy": {"connectionTimeoutInSeconds": 0, "responseTimeoutInSeconds": 0},
                "type": "str",
            },
            api_version="2025-02-02-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_app_resiliency_update(self, resource_group):
        response = self.client.app_resiliency.update(
            resource_group_name=resource_group.name,
            app_name="str",
            name="str",
            resiliency_envelope={
                "circuitBreakerPolicy": {"consecutiveErrors": 0, "intervalInSeconds": 0, "maxEjectionPercent": 0},
                "httpConnectionPool": {"http1MaxPendingRequests": 0, "http2MaxRequests": 0},
                "httpRetryPolicy": {
                    "errors": ["str"],
                    "headers": [
                        {
                            "exactMatch": "str",
                            "header": "str",
                            "prefixMatch": "str",
                            "regexMatch": "str",
                            "suffixMatch": "str",
                        }
                    ],
                    "httpStatusCodes": [0],
                    "initialDelayInMilliseconds": 0,
                    "maxIntervalInMilliseconds": 0,
                    "maxRetries": 0,
                },
                "id": "str",
                "name": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tcpConnectionPool": {"maxConnections": 0},
                "tcpRetryPolicy": {"maxConnectAttempts": 0},
                "timeoutPolicy": {"connectionTimeoutInSeconds": 0, "responseTimeoutInSeconds": 0},
                "type": "str",
            },
            api_version="2025-02-02-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_app_resiliency_delete(self, resource_group):
        response = self.client.app_resiliency.delete(
            resource_group_name=resource_group.name,
            app_name="str",
            name="str",
            api_version="2025-02-02-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_app_resiliency_get(self, resource_group):
        response = self.client.app_resiliency.get(
            resource_group_name=resource_group.name,
            app_name="str",
            name="str",
            api_version="2025-02-02-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_app_resiliency_list(self, resource_group):
        response = self.client.app_resiliency.list(
            resource_group_name=resource_group.name,
            app_name="str",
            api_version="2025-02-02-preview",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...



# coding=utf-8
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient
from corehttp.runtime.pipeline import PipelineResponse
from corehttp.utils import case_insensitive_dict

from ... import models as _models
from ..._operations._operations import (
    build_not_discriminated_get_valid_request,
    build_not_discriminated_post_valid_request,
    build_not_discriminated_put_valid_request,
)
from ..._utils.model_base import SdkJSONEncoder, _deserialize
from ..._utils.utils import ClientMixinABC
from .._configuration import NotDiscriminatedClientConfiguration

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class _NotDiscriminatedClientOperationsMixin(
    ClientMixinABC[AsyncPipelineClient[HttpRequest, AsyncHttpResponse], NotDiscriminatedClientConfiguration]
):

    @overload
    async def post_valid(
        self, input: _models.Siamese, *, content_type: str = "application/json", **kwargs: Any
    ) -> None:
        """post_valid.

        :param input: Required.
        :type input: ~typetest.model.notdiscriminated.models.Siamese
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    async def post_valid(self, input: JSON, *, content_type: str = "application/json", **kwargs: Any) -> None:
        """post_valid.

        :param input: Required.
        :type input: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    async def post_valid(self, input: IO[bytes], *, content_type: str = "application/json", **kwargs: Any) -> None:
        """post_valid.

        :param input: Required.
        :type input: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    async def post_valid(self, input: Union[_models.Siamese, JSON, IO[bytes]], **kwargs: Any) -> None:
        """post_valid.

        :param input: Is one of the following types: Siamese, JSON, IO[bytes] Required.
        :type input: ~typetest.model.notdiscriminated.models.Siamese or JSON or IO[bytes]
        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(input, (IOBase, bytes)):
            _content = input
        else:
            _content = json.dumps(input, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_not_discriminated_post_valid_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client.pipeline.run(  # type: ignore
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    async def get_valid(self, **kwargs: Any) -> _models.Siamese:
        """get_valid.

        :return: Siamese. The Siamese is compatible with MutableMapping
        :rtype: ~typetest.model.notdiscriminated.models.Siamese
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.Siamese] = kwargs.pop("cls", None)

        _request = build_not_discriminated_get_valid_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client.pipeline.run(  # type: ignore
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.Siamese, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def put_valid(
        self, input: _models.Siamese, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.Siamese:
        """put_valid.

        :param input: Required.
        :type input: ~typetest.model.notdiscriminated.models.Siamese
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Siamese. The Siamese is compatible with MutableMapping
        :rtype: ~typetest.model.notdiscriminated.models.Siamese
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    async def put_valid(self, input: JSON, *, content_type: str = "application/json", **kwargs: Any) -> _models.Siamese:
        """put_valid.

        :param input: Required.
        :type input: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Siamese. The Siamese is compatible with MutableMapping
        :rtype: ~typetest.model.notdiscriminated.models.Siamese
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    async def put_valid(
        self, input: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.Siamese:
        """put_valid.

        :param input: Required.
        :type input: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Siamese. The Siamese is compatible with MutableMapping
        :rtype: ~typetest.model.notdiscriminated.models.Siamese
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    async def put_valid(self, input: Union[_models.Siamese, JSON, IO[bytes]], **kwargs: Any) -> _models.Siamese:
        """put_valid.

        :param input: Is one of the following types: Siamese, JSON, IO[bytes] Required.
        :type input: ~typetest.model.notdiscriminated.models.Siamese or JSON or IO[bytes]
        :return: Siamese. The Siamese is compatible with MutableMapping
        :rtype: ~typetest.model.notdiscriminated.models.Siamese
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Siamese] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(input, (IOBase, bytes)):
            _content = input
        else:
            _content = json.dumps(input, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_not_discriminated_put_valid_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client.pipeline.run(  # type: ignore
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models.Siamese, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore
